/* -------------------------------------------------------------------------- */
/*                                  Libraries                                 */
/* -------------------------------------------------------------------------- */
import React from "react";
/* -------------------------------------------------------------------------- */
/*                                    Types                                   */
/* -------------------------------------------------------------------------- */
import type { UserStore } from "../domain/userStore";
import type { User } from "../domain/userEntity";
/* -------------------------------------------------------------------------- */
/*                                    Store                                   */
/* -------------------------------------------------------------------------- */
import type { UserStoreState } from "./userReducer";
import { RootState, useAppDispatch, useAppSelector } from "store";
/* -------------------------------------------------------------------------- */
/*                                   Actions                                  */
/* -------------------------------------------------------------------------- */
import {
  getUserAction,
  setUserAction,
  updateUserAction,
} from "./userActions";
/* -------------------------------------------------------------------------- */
/*                                 User slice                                 */
/* -------------------------------------------------------------------------- */
const userSelector = (state: RootState): UserStoreState => state.user;

const useUserStore = (): UserStore => {
  const { user, isLoading, isUpdating } = useAppSelector(userSelector);
  
  const dispatch = useAppDispatch();

  const setUser = React.useCallback(
    (user: User) => setUserAction(user)(dispatch),
    [dispatch]
  );

  const loadUser = React.useCallback(
    () => getUserAction()(dispatch),
    [dispatch]
  );
  
  const updateUser = React.useCallback(
    (user: User) => updateUserAction(user)(dispatch),
    [dispatch]
  );


  return {
    user,
    isLoading,
    isUpdating,
    updateUser,
    setUser,
    loadUser,
  };
};

export { useUserStore };
